#include <Arduino.h>
#include <OneWire.h>

#include "DS2423.h"

#define MY_DEBUG
#define MY_RADIO_RF24

#define MY_RF24_PA_LEVEL RF24_PA_MAX
#define RF24_DATARATE      RF24_250KBPS

#define CHILD_ID_COUNT_A 0
#define CHILD_ID_COUNT_B 1
#define CHILD_ID_COUNT_AB 2

const uint8_t ONE_WIRE_PIN = 3;
uint16_t const count_interval = 5000;
static uint32_t previousMillis;
bool flag = 0;

uint8_t DS2423_address[] = { 0x1d, 0xbb, 0x30, 0x0a, 0x00, 0x00, 0x00, 0xa3 };

OneWire ow(ONE_WIRE_PIN);
DS2423 ds2423(&ow, DS2423_address);
#define MY_NODE_ID 21
#include <MySensors.h>

MyMessage msg_c_A(CHILD_ID_COUNT_A, V_VAR1);
MyMessage msg_c_B(CHILD_ID_COUNT_B, V_VAR2);
MyMessage msg_c_AB(CHILD_ID_COUNT_AB, V_VAR3);


void presentation()
{
  sendSketchInfo("Counter Sensor", "1.0");
  present(CHILD_ID_COUNT_A, S_CUSTOM, "CounterA sensor");
  wait(100);
  present(CHILD_ID_COUNT_B, S_CUSTOM, "CounterB sensor");
  wait(100);
  present(CHILD_ID_COUNT_AB, S_CUSTOM, "SplitCounterAB sensor");
}


void setup() {
  ds2423.begin(DS2423_COUNTER_A | DS2423_COUNTER_B);

  previousMillis = millis();
}


void loop() {
  if (millis() - previousMillis > count_interval) {
    if (flag == 1) {
      ds2423_read_ver1 ();
      flag = 0;
      previousMillis = millis();
    }
    else {
      ds2423_read_ver2 ();
      flag = 1;
      previousMillis = millis();
    }
  }
}

void receive(const MyMessage & message) {

  if (message.type == V_VAR1) {
    if (message.sensor == CHILD_ID_COUNT_A) {
      TRANSPORT_DEBUG(PSTR("MyS_DS2423: Count A Delivered\n"));
    }
  }
  if (message.type == V_VAR2) {
    if (message.sensor == CHILD_ID_COUNT_B) {
      TRANSPORT_DEBUG(PSTR("MyS_DS2423: Send: Count B Delivered\n"));
    }
  }
  if (message.type == V_VAR3) {
    if (message.sensor == CHILD_ID_COUNT_AB) {
      TRANSPORT_DEBUG(PSTR("MyS_DS2423: Send: Counts AB Delivered\n"));
    }
  }
}

void ds2423_read_ver1 () {
  ds2423.update();
  uint8_t counA = ds2423.getCount(DS2423_COUNTER_A / 430); // Что то тут не так - считает в обратную сторону
  uint8_t counB = ds2423.getCount(DS2423_COUNTER_B / 430);
 
  if (ds2423.isError()) {
    TRANSPORT_DEBUG(PSTR("MyS_DS2423: Error reading counter\n"));
    // Что то делаем или отправляем
  } else {
    String A = (String(counA) + "," + String(counB));
    int str_len = A.length() + 1;
    char my_countAB[str_len];
    A.toCharArray(my_countAB, str_len);
    send(msg_c_AB.set(my_countAB), true);
    wait(1000, 1, 26); // delay = 1000, comand = set = 1, tape = V_VAR3 = 26
    TRANSPORT_DEBUG(PSTR("MyS: Count A = %d, Count B =  %d\n"), counA, counB);
  }
}

void ds2423_read_ver2 () {
  ds2423.update();
  uint8_t counA = ds2423.getCount(DS2423_COUNTER_A / 430);
  uint8_t counB = ds2423.getCount(DS2423_COUNTER_B / 430);

  if (ds2423.isError()) {
    TRANSPORT_DEBUG(PSTR("MyS_DS2423: Error reading counter\n"));
    // Что то делаем или отправляем
  } else {
    send(msg_c_A.set(counA), true);
    wait(1000, 1, 24); // delay = 1000, comand = set = 1, tape = V_VAR1 = 24
    send(msg_c_B.set(counB), true);
    wait(1000, 1, 25); // delay = 1000, comand = set = 1, tape = V_VAR2 = 25
    TRANSPORT_DEBUG(PSTR("MyS: Count A = %d, Count B =  %d\n"), counA, counB);
  }
}

 

